# frozen-string-literal: true

# Copyright (C) 2020 Thomas Baron
#
# This file is part of term_utils.
#
# term_utils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# term_utils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with term_utils.  If not, see <https://www.gnu.org/licenses/>.

module TermUtils
  # The Argument Parser module provides a way to parse arguments.
  module AP
    # NoSuchValueError.
    class NoSuchValueError < StandardError
      def initialize(msg)
        super
      end
    end

    # ParseError.
    class ParseError < StandardError
      def initialize(msg)
        super
      end
    end

    # SyntaxError.
    class SyntaxError < StandardError
      def initialize(msg)
        super
      end
    end

    # Creates a new Syntax.
    # @return [TermUtils::AP::Syntax]
    def self.create_syntax(&block)
      new_syntax = TermUtils::AP::Syntax.new
      block.call(new_syntax) if block
      new_syntax
    end

    # Parses a given list of arguments.
    # @param syntax [Syntax]
    # @param arguments [Array<String>]
    # @param opts [Hash]
    # @option opts [Boolean] :strict Whether the Syntax must be considered strict.
    # @return [Result]
    # @raise [ParseError]
    # @raise [SyntaxError]
    def self.parse_arguments(syntax, arguments, opts = {}, &block)
      TermUtils::AP::Parser.new.parse_arguments(syntax, arguments, opts, &block)
    end
  end
end

require 'term_utils/ap/article'
require 'term_utils/ap/flag'
require 'term_utils/ap/parameter'
require 'term_utils/ap/syntax'
require 'term_utils/ap/result'
require 'term_utils/ap/parser'
